﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.CodeAnalysis;

namespace VsxFactory.ExtensionManager
{
    public class PackageEqualityComparer : IEqualityComparer<DataServicePackage>
    {
        [SuppressMessage("Microsoft.Security", "CA2104:DoNotDeclareReadOnlyMutableReferenceTypes", Justification = "This type isn't mutable")]
        public static readonly PackageEqualityComparer IdAndVersion = new PackageEqualityComparer((x, y) => x.Name.Equals(y.Name, StringComparison.OrdinalIgnoreCase) &&
                                                                                                                    x.Version.Equals(y.Version),
                                                                                                   x => x.Name.GetHashCode() ^ x.Version.GetHashCode());

        [SuppressMessage("Microsoft.Security", "CA2104:DoNotDeclareReadOnlyMutableReferenceTypes", Justification = "This type isn't mutable")]
        public static readonly PackageEqualityComparer Id = new PackageEqualityComparer((x, y) => x.Name.Equals(y.Name, StringComparison.OrdinalIgnoreCase),
                                                                                         x => x.Name.GetHashCode());

        private readonly Func<DataServicePackage, DataServicePackage, bool> _equals;
        private readonly Func<DataServicePackage, int> _getHashCode;

        private PackageEqualityComparer(Func<DataServicePackage, DataServicePackage, bool> equals, Func<DataServicePackage, int> getHashCode)
        {
            _equals = equals;
            _getHashCode = getHashCode;
        }

        public bool Equals(DataServicePackage x, DataServicePackage y)
        {
            return _equals(x, y);
        }

        public int GetHashCode(DataServicePackage obj)
        {
            return _getHashCode(obj);
        }
    }
}
